# Mechanical Turk Qualifications

[Qualifications](https://docs.aws.amazon.com/AWSMechTurk/latest/AWSMechanicalTurkRequester/Concepts_QualificationsArticle.html)
are attributes associated with a worker that can be used to filter who is able
to accept, preview, or even discover a hit.  There are several kinds of qualifications
available
1.  [Premium and Masters](https://requester.mturk.com/pricing) (additional cost per assignment)
2.  Autogenerated (approval rate, location, etc.)
3.  Custom (requestor created and managed)

## Creating custom qualification types

*Qualification types are generated per environment (sandbox or production).  You
will need to manage them separately for production/testing.*

Stumpf provides an API for generating qualification types.  For example,
```
POST /api/v1/qualification-type
{
    "sandbox": true,
    "name": "My custom qualification",
    "description": "A description of the qualification",
    "keywords": "arity,keyword1,keyword2"
}
```
**The name, description, and keywords are visible to the workers.**  If the
qualification is taylored to a specific application, it is useful to use the
application name as one of the keywords.  This endpoint will return both an
internal stumpf ID (UUID) and a mechanical turk ID (mturk_qt_id).  The
mechanical turk ID is what should be used when creating tasks with
qualification requirements.

## Assigning a user a qualification score

When managing a custom qualification type, it is the app author's
responsibility to generate scores for workers and post them back mturk.  See
the [boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/mturk.html#MTurk.Client.associate_qualification_with_worker)
for details.

### Creating a hit as a qualification test

When ground truth for an app is available, it is possible to generate a HIT
that is used as a qualification test.  For this case, you can use the optional
`qualification_for_id` parameter when creating the hit to qualify worker's
automatically inside of a custom pipeline task.  There is no automatic
way to link a worker to the qualification test when submitting hits in this
way.  The hit creator must ensure that qualification hits are created and
available for workers for the duration of the actual hits.  In addition, the
qualification hits should generate another qualification to prevent workers
from taking the qualification tests multiple times.  For these reasons, this
kind of custom qualfication testing is not recommended.

### Scoring workers by random sampling

A more robust form of quality control is to include in each batch of hits a
certain percentage of tasks with known answers.  An assignment status change
handler can be created that detects assignments coming in with known answers.
The handler can then score the answer and update a custom qualification.  This
could either be a score that is not allowed to dip below a certain threshold or
blacklist of known bad performers.

There is no way to ensure that a worker accepts the ground truth hits.  You must
rely on random chance to ensure that workers are being occasionally tested.  This
is not reliable initially when submitting a small number of hits, but it will
improve as the number of generate hits gets large.

**You should take care not to expose the fact that a hit is used as a
qualification test to the worker.**  Workers have access to the task name,
description, and arguments.  In addition, because the models are transmitted
to the worker during authentication, anything set on the task, assignment,
or video tables are theoretically discoverable by the worker.  In the future,
we may need to provide a mechanism for filtering out information sent over
the mturk API to truly hide this information.

## Generating HITS with qualification requirements

The following are useful qualifications provided by mturk that do not incur any
additional costs:

|           ID           |         Name          |  Value  |
|------------------------|-----------------------|---------|
| `000000000000000000L0` | HIT approval rate (%) | 0 - 100 |
| `00000000000000000040` | Total approved HITs   | >= 0    |
| `00000000000000000071` | Location              | [Locale object](https://docs.aws.amazon.com/AWSMechTurk/latest/AWSMturkAPI/ApiReference_LocaleDataStructureArticle.html) |

These qualification types (plus any custom qualification types) can be used in
the `qualification_requirements` list accepted by the `POST /api/v1/task`
endpoint.  For example, the following list will limit the task to workers that:
1.  Have completed 1000 hits.
2.  Have an approval rate > 95%
3.  Live in the US or Canada
4.  Are not in a custom blacklist qualification.

```json
[
    {
        "QualificationTypeId": "00000000000000000040",
        "Comparator": "GreaterThan",
        "IntegerValues": [ 1000 ],
        "ActionsGuarded": "DiscoverPreviewAndAccept"
    }, {
        "QualificationTypeId": "000000000000000000L0",
        "Comparator": "GreaterThan",
        "IntegerValues": [ 95 ],
        "ActionsGuarded": "DiscoverPreviewAndAccept"
    }, {
        "QualificationTypeId": "00000000000000000071",
        "Comparator": "In",
        "LocaleValues": [{"Country": "US"}, {"Country": "CA"}],
        "ActionsGuarded": "DiscoverPreviewAndAccept"
    }, {
        "QualificationTypeId": "....",
        "Comparator": "DoesNotExist",
        "ActionsGuarded": "DiscoverPreviewAndAccept"
    }
]
```
