import logging
import os

import sentry_sdk
from sentry_sdk.integrations.celery import CeleryIntegration
from sentry_sdk.integrations.logging import LoggingIntegration
from sentry_sdk.integrations.sqlalchemy import SqlalchemyIntegration

from stumpf.celery.utils import make_celery

if os.getenv('CELERY_SENTRY_DSN', False):
    # Initialize sentry for celery workers.  The flask sentry handler
    # does not initialize until after the worker is started.
    sentry_sdk.init(dsn=os.getenv('CELERY_SENTRY_DSN'),
                    send_default_pii=True,
                    integrations=[CeleryIntegration(),
                                  LoggingIntegration(event_level=logging.WARNING),
                                  SqlalchemyIntegration()])
app = make_celery()
