import os

from celery.schedules import crontab
from dotenv import load_dotenv


def default_config(app):
    load_dotenv(os.getenv('DOTENV_PATH'))
    app.conf.result_backend = f'redis://{os.environ.get("BROKER_URL", "")}:6379/0'
    app.conf.broker_url = f'redis://{os.environ.get("BROKER_URL", "")}:6379/0'
    app.conf.redis_socket_connect_timeout = 2
    app.conf.timezone = 'America/New_York'
    app.conf.task_routes = ([
        ('stumpf.celery.tasks.create_hit_task', {'queue': 'hits'}),
    ],)

    app.conf.beat_schedule = {
        'abandoned-expired-kwit-assignments': {
            'task': 'stumpf.celery.tasks.abandon_expired_kwit_assignments',
            'schedule': crontab(minute='*'),
        },
        'clean-expired-tokens': {
            'task': 'stumpf.celery.tasks.clean_expired_tokens',
            'schedule': crontab(hour='0', minute='0'),
        }
    }
    app.conf.imports += ('stumpf.celery.tasks',)
