from functools import wraps

from botocore.exceptions import ClientError
from flask import current_app, jsonify, request


def public_endpoint(func):
    func.is_public = True
    return func


def is_public():
    if request.endpoint not in current_app.view_functions:
        return False
    func = current_app.view_functions[request.endpoint]
    return getattr(func, 'is_public', False)


def handle_boto_client_errors(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except ClientError as e:
            return jsonify(e.response['Error']), 400
    return wrapper
