from flask import Blueprint, current_app, redirect, request, Response, session, url_for
from flask_cors import cross_origin
from flask_login import logout_user
import redis

from stumpf.decorators import public_endpoint
from stumpf.models import db, Token

bp = Blueprint('index', __name__)


@bp.route('/logout')
@cross_origin()
@public_endpoint
def logout():
    token = request.cookies.get('stumpf-token')
    if token:
        Token.query.filter_by(token=token).delete()
        db.session.commit()
    logout_user()
    response = Response()
    response.set_cookie('stumpf-token', '')
    return response


@bp.route('/login')
@public_endpoint
def login():
    next_url = request.args.get('next', '/')
    session['oauth-redirect-destination'] = next_url
    return redirect(url_for('google.login'))


@bp.route('/health')
@public_endpoint
def health():
    redis.Redis(current_app.config['BROKER_URL']).ping()
    db.session.execute('SELECT 1')
    return '', 200
