from flask import abort, g, request
from flask_login import login_user
import sentry_sdk

from stumpf.decorators import is_public
from stumpf.models import MturkToken, Token


def before_all_requests():
    if is_public() or request.method == 'OPTIONS' or request.routing_exception:
        g.bypass_auth_middlewares = True
    else:
        g.bypass_auth_middlewares = False


def mturk_auth_middleware(*args, **kwargs):
    if not g.bypass_auth_middlewares:
        token = MturkToken.query.filter_by(
            expired=False, token=request.headers.get('X-Stumpf-Mturk-Token')).first()

        if token:
            g.assignment_id = token.assignment_id

            # Add assignment ID to sentry context for debugging
            with sentry_sdk.configure_scope() as scope:
                scope.set_tag('assignment_id', str(g.assignment_id))
        else:
            abort(401)


def json_content_middleware(*args, **kwargs):
    if request.content_type != 'application/json':
        abort(415)


def google_auth_middleware(*args, **kwargs):
    from stumpf.auth import login_manager

    if not g.bypass_auth_middlewares:
        user = Token.user_from_token(request.headers.get('X-Stumpf-Token'))

        if user is None:
            login_manager.unauthorized()
        else:
            login_user(user)
