from datetime import datetime, timedelta

import pytest

from stumpf.fakes import fake
from stumpf.models import db


@pytest.fixture
def example_assignments(app, annotation_app):
    with app.app_context():
        user = fake.stumpf_user()
        db.session.add(user)
        db.session.commit()

        # Setup 2 KWITs with a 5 minute assignment time
        tasks = []
        for _ in range(2):
            task = fake.stumpf_task(annotation_app.id, user.id, status='Unassignable')
            task.type_ = 'KWIT'
            task.assignment_duration_in_seconds = 300
            db.session.add(task)
            db.session.commit()
            tasks.append(task)

        # Create an assignment from 10 minutes ago that has been accepted
        expired_assignment = fake.stumpf_assignment(tasks[0].id)
        expired_assignment.created = datetime.utcnow() - timedelta(minutes=10)
        expired_assignment.status = 'Accepted'
        db.session.add(expired_assignment)
        db.session.commit()

        # Create an assignment that has just been accepted
        fresh_assignment = fake.stumpf_assignment(tasks[1].id)
        fresh_assignment.status = 'Accepted'
        db.session.add(fresh_assignment)
        db.session.commit()

        yield expired_assignment, fresh_assignment


def test_kwit_assignment_expiration(app, example_assignments):
    from stumpf.celery.tasks import abandon_expired_kwit_assignments

    expired_assignment, fresh_assignment = example_assignments

    assert expired_assignment.status == 'Accepted'
    assert expired_assignment.task.status == 'Unassignable'
    assert fresh_assignment.status == 'Accepted'
    assert fresh_assignment.task.status == 'Unassignable'

    abandon_expired_kwit_assignments()

    db.session.refresh(expired_assignment)
    db.session.refresh(fresh_assignment)

    assert expired_assignment.status == 'Abandoned'
    assert expired_assignment.task.status == 'Assignable'
    assert fresh_assignment.status == 'Accepted'
    assert fresh_assignment.task.status == 'Unassignable'
