from mock import Mock

from stumpf.models import db
from stumpf.pipelines.filters import assignment_filter, task_filter


def test_should_run_assignment(app, assignment, task):
    with app.app_context():
        task.sandbox = False
        task.status = 'Assignable'
        assignment.status = 'Submitted'

        db.session.add_all([task, assignment])
        db.session.commit()

        decorator = assignment_filter(task.annotation_app_id)
        mock = Mock()
        handler = decorator(mock)
        handler(None, assignment.id, None)

        assert mock.call_count == 1


def test_sandbox_assignment(app, assignment, task):
    with app.app_context():
        task.sandbox = True
        task.status = 'Assignable'
        assignment.status = 'Submitted'

        db.session.add_all([task, assignment])
        db.session.commit()

        decorator = assignment_filter(task.annotation_app_id)
        mock = Mock()
        handler = decorator(mock)
        handler(None, assignment.id, None)

        assert mock.call_count == 0


def test_should_run_task(app, task):
    with app.app_context():
        task.sandbox = False
        task.status = 'Reviewable'

        db.session.add(task)
        db.session.commit()

        decorator = task_filter(task.annotation_app_id)
        mock = Mock()
        handler = decorator(mock)
        handler(None, task.id, None)

        assert mock.call_count == 1


def test_sandbox_task(app, task):
    with app.app_context():
        task.sandbox = True
        task.status = 'Reviewable'

        db.session.add(task)
        db.session.commit()

        decorator = task_filter(task.annotation_app_id)
        mock = Mock()
        handler = decorator(mock)
        handler(None, task.id, None)

        assert mock.call_count == 0


def test_type_filter(app, task):
    with app.app_context():
        task.sandbox = False
        task.status = 'Reviewable'
        task.type_ = 'KWIT'

        db.session.add(task)
        db.session.commit()

        decorator = task_filter(task.annotation_app_id, types=('HIT',))
        mock = Mock()
        handler = decorator(mock)
        handler(None, task.id, None)

        assert mock.call_count == 0


def test_no_pipeline_filter(app, task):
    with app.app_context():
        task.sandbox = False
        task.status = 'Reviewable'
        task.keywords += ',no-pipeline'

        db.session.add(task)
        db.session.commit()

        decorator = task_filter(task.annotation_app_id)
        mock = Mock()
        handler = decorator(mock)
        handler(None, task.id, None)

        assert mock.call_count == 0
