import pytest

from stumpf.fakes import fake
from stumpf.models import db


@pytest.fixture
def tasks(app, annotation_app):
    tasks_ = []

    with app.app_context():
        user = fake.stumpf_user()
        db.session.add(user)
        db.session.commit()

        for i in range(3):
            task = fake.stumpf_task(annotation_app.id, user.id)
            task.sandbox = bool(i == 0)
            task.status = 'Assignable' if i == 0 else 'Unassignable'
            tasks_.append(task)
            db.session.add(task)
        db.session.commit()

        yield tasks_


def test_task_listing(app, auth, tasks):
    with app.test_client() as client:
        resp = client.get('/api/v1/task', query_string={
            'sandbox': 'true',
            'status': 'Assignable'
        }, headers={
            'X-Stumpf-Token': auth[1].token
        })
        assert resp.status_code == 200, resp.json
        assert len(resp.json) == 1

        resp = client.get('/api/v1/task', headers={
            'X-Stumpf-Token': auth[1].token
        })
        assert resp.status_code == 200, resp.json
        assert len(resp.json) == 3
